﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Collections.ObjectModel;
using System.ComponentModel;
using BuzzGUI.Interfaces;
using BuzzGUI.Common;
using BuzzGUI.Common.InterfaceExtensions;
using System.Windows.Input;

namespace Jeskola.Live
{
	public class LiveVM : INotifyPropertyChanged
	{
		public ObservableCollection<TrackVM> Tracks { get; private set; }
		public int Bar { get { return barParameter.GetValue(0);	} }

		IMachine machine;
		ISong song;
		IParameter barParameter;

		public LiveVM(IMachine machine)
		{
			this.machine = machine;
			this.song = machine.Graph.Buzz.Song;

			barParameter = machine.GetParameter("Bar");
			barParameter.SubscribeEvents(0, BarChanged, null);

			song.SequenceAdded += song_SequenceAdded;
			song.SequenceChanged += song_SequenceChanged;
			song.SequenceRemoved += song_SequenceRemoved;

			Tracks = new ObservableCollection<TrackVM>();
			for (int i = 0; i < song.Sequences.Count; i++) song_SequenceAdded(i);

		}

		public void Release()
		{
			barParameter.UnsubscribeEvents(0, BarChanged, null);

			song.SequenceAdded -= song_SequenceAdded;
			song.SequenceChanged -= song_SequenceChanged;
			song.SequenceRemoved -= song_SequenceRemoved;
		}

		void BarChanged(IParameter p, int track)
		{
			PropertyChanged.Raise(this, "Bar");
		}

		void song_SequenceAdded(int index)
		{
			Tracks.Add(new TrackVM(this, song.Sequences[index]));
		}

		void song_SequenceChanged(int index)
		{
			Tracks[index].Release();
			Tracks[index] = new TrackVM(this, song.Sequences[index]);
		}

		void song_SequenceRemoved(int index)
		{
			Tracks[index].Release();
			Tracks.RemoveAt(index);
		}

		public void Update()
		{
			foreach (var t in Tracks)
				t.Update();
		}


		public event PropertyChangedEventHandler PropertyChanged;

	}
}
